import * as Backbone from 'backbone';
import { ManagerBase, IClassicComm, IWidgetRegistryData, WidgetModel, WidgetView, IStateOptions } from '@jupyter-widgets/base';
import { IDisposable } from '@phosphor/disposable';
import { Widget } from '@phosphor/widgets';
import { IRenderMimeRegistry } from '@jupyterlab/rendermime';
import { Kernel } from '@jupyterlab/services';
import { ISignal } from '@phosphor/signaling';
/**
 * A widget manager that returns phosphor widgets.
 */
export declare class WidgetManager extends ManagerBase<Widget> implements IDisposable {
    constructor(kernel: Kernel.IKernel, rendermime: IRenderMimeRegistry);
    _handleKernelStatusChange(args: Kernel.Status): void;
    /**
     * Restore widgets from kernel and saved state.
     */
    restoreWidgets(): Promise<void>;
    /**
     * Disconnect the widget manager from the kernel, setting each model's comm
     * as dead.
     */
    disconnect(): void;
    _loadFromKernel(): Promise<void>;
    /**
     * Return a phosphor widget representing the view
     */
    display_view(msg: any, view: Backbone.View<Backbone.Model>, options: any): Promise<Widget>;
    /**
     * Create a comm.
     */
    _create_comm(target_name: string, model_id: string, data?: any, metadata?: any, buffers?: ArrayBuffer[] | ArrayBufferView[]): Promise<IClassicComm>;
    /**
     * Get the currently-registered comms.
     */
    _get_comm_info(): Promise<any>;
    /**
     * Get whether the manager is disposed.
     *
     * #### Notes
     * This is a read-only property.
     */
    readonly isDisposed: boolean;
    /**
     * Dispose the resources held by the manager.
     */
    dispose(): void;
    /**
     * Load a class and return a promise to the loaded object.
     */
    protected loadClass(className: string, moduleName: string, moduleVersion: string): Promise<typeof WidgetModel | typeof WidgetView>;
    readonly kernel: Kernel.IKernel;
    readonly rendermime: IRenderMimeRegistry;
    /**
     * A signal emitted when state is restored to the widget manager.
     *
     * #### Notes
     * This indicates that previously-unavailable widget models might be available now.
     */
    readonly restored: ISignal<this, void>;
    /**
     * Whether the state has been restored yet or not.
     */
    readonly restoredStatus: boolean;
    register(data: IWidgetRegistryData): void;
    /**
     * Get a model
     *
     * #### Notes
     * Unlike super.get_model(), this implementation always returns a promise and
     * never returns undefined. The promise will reject if the model is not found.
     */
    get_model(model_id: string): Promise<WidgetModel>;
    /**
     * Register a widget model.
     */
    register_model(model_id: string, modelPromise: Promise<WidgetModel>): void;
    /**
     * Close all widgets and empty the widget state.
     * @return Promise that resolves when the widget state is cleared.
     */
    clear_state(): Promise<void>;
    /**
     * Synchronously get the state of the live widgets in the widget manager.
     *
     * This includes all of the live widget models, and follows the format given in
     * the @jupyter-widgets/schema package.
     *
     * @param options - The options for what state to return.
     * @returns Promise for a state dictionary
     */
    get_state_sync(options?: IStateOptions): {
        version_major: number;
        version_minor: number;
        state: {
            [key: string]: any;
        };
    };
    private _handleCommOpen;
    private _kernel;
    private _registry;
    private _rendermime;
    private _restored;
    private _restoredStatus;
    private _modelsSync;
}
