"use strict";
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const base_1 = require("@jupyter-widgets/base");
const signaling_1 = require("@phosphor/signaling");
const semver_1 = require("semver");
const semvercache_1 = require("@jupyter-widgets/jupyterlab-manager/lib/semvercache");
const manager_1 = require("@jupyter-widgets/jupyterlab-manager/lib/manager");
/**
 * A widget manager that returns phosphor widgets.
 */
class WidgetManager extends base_1.ManagerBase {
    constructor(kernel, rendermime) {
        super();
        this._registry = new semvercache_1.SemVerCache();
        this._restored = new signaling_1.Signal(this);
        this._restoredStatus = false;
        this._modelsSync = new Map();
        this._kernel = kernel;
        this._rendermime = rendermime;
        // Set _handleCommOpen so `this` is captured.
        this._handleCommOpen = (comm, msg) => __awaiter(this, void 0, void 0, function* () {
            let oldComm = new base_1.shims.services.Comm(comm);
            yield this.handle_comm_open(oldComm, msg);
        });
        kernel.statusChanged.connect((sender, args) => {
            this._handleKernelStatusChange(args);
        });
        kernel.registerCommTarget(this.comm_target_name, this._handleCommOpen);
        this.restoreWidgets();
    }
    _handleKernelStatusChange(args) {
        switch (args) {
            case 'connected':
                this.restoreWidgets();
                break;
            case 'restarting':
                this.disconnect();
                break;
            default:
        }
    }
    /**
     * Restore widgets from kernel and saved state.
     */
    restoreWidgets() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._loadFromKernel();
            this._restoredStatus = true;
            this._restored.emit();
        });
    }
    /**
     * Disconnect the widget manager from the kernel, setting each model's comm
     * as dead.
     */
    disconnect() {
        super.disconnect();
        this._restoredStatus = false;
    }
    _loadFromKernel() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.kernel) {
                return;
            }
            yield this.kernel.ready;
            const comm_ids = yield this._get_comm_info();
            // For each comm id, create the comm, and request the state.
            const widgets_info = yield Promise.all(Object.keys(comm_ids).map((comm_id) => __awaiter(this, void 0, void 0, function* () {
                const comm = yield this._create_comm(this.comm_target_name, comm_id);
                const update_promise = new Promise((resolve, reject) => {
                    comm.on_msg((msg) => {
                        base_1.put_buffers(msg.content.data.state, msg.content.data.buffer_paths, msg.buffers);
                        // A suspected response was received, check to see if
                        // it's a state update. If so, resolve.
                        if (msg.content.data.method === 'update') {
                            resolve({
                                comm: comm,
                                msg: msg
                            });
                        }
                    });
                });
                comm.send({
                    method: 'request_state'
                }, this.callbacks(undefined));
                return yield update_promise;
            })));
            // We put in a synchronization barrier here so that we don't have to
            // topologically sort the restored widgets. `new_model` synchronously
            // registers the widget ids before reconstructing their state
            // asynchronously, so promises to every widget reference should be available
            // by the time they are used.
            yield Promise.all(widgets_info.map((widget_info) => __awaiter(this, void 0, void 0, function* () {
                const content = widget_info.msg.content;
                yield this.new_model({
                    model_name: content.data.state._model_name,
                    model_module: content.data.state._model_module,
                    model_module_version: content.data.state._model_module_version,
                    comm: widget_info.comm,
                }, content.data.state);
            })));
        });
    }
    /**
     * Return a phosphor widget representing the view
     */
    display_view(msg, view, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return view.pWidget || new manager_1.BackboneViewWrapper(view);
        });
    }
    /**
     * Create a comm.
     */
    _create_comm(target_name, model_id, data, metadata, buffers) {
        return __awaiter(this, void 0, void 0, function* () {
            let comm = this._kernel.connectToComm(target_name, model_id);
            if (data || metadata) {
                comm.open(data, metadata, buffers);
            }
            return new base_1.shims.services.Comm(comm);
        });
    }
    /**
     * Get the currently-registered comms.
     */
    _get_comm_info() {
        return __awaiter(this, void 0, void 0, function* () {
            const reply = yield this._kernel.requestCommInfo({ target: this.comm_target_name });
            if (reply.content.status === 'ok') {
                return reply.content.comms;
            }
            else {
                return {};
            }
        });
    }
    /**
     * Get whether the manager is disposed.
     *
     * #### Notes
     * This is a read-only property.
     */
    get isDisposed() {
        return this._kernel === null;
    }
    /**
     * Dispose the resources held by the manager.
     */
    dispose() {
        if (this.isDisposed) {
            return;
        }
        this._kernel = null;
    }
    /**
     * Load a class and return a promise to the loaded object.
     */
    loadClass(className, moduleName, moduleVersion) {
        return __awaiter(this, void 0, void 0, function* () {
            // Special-case the Jupyter base and controls packages. If we have just a
            // plain version, with no indication of the compatible range, prepend a ^ to
            // get all compatible versions. We may eventually apply this logic to all
            // widget modules. See issues #2006 and #2017 for more discussion.
            if ((moduleName === '@jupyter-widgets/base'
                || moduleName === '@jupyter-widgets/controls')
                && semver_1.valid(moduleVersion)) {
                moduleVersion = `^${moduleVersion}`;
            }
            const mod = this._registry.get(moduleName, moduleVersion);
            if (!mod) {
                throw new Error(`Module ${moduleName}, semver range ${moduleVersion} is not registered as a widget module`);
            }
            let module;
            if (typeof mod === 'function') {
                module = yield mod();
            }
            else {
                module = yield mod;
            }
            const cls = module[className];
            if (!cls) {
                throw new Error(`Class ${className} not found in module ${moduleName}`);
            }
            return cls;
        });
    }
    get kernel() {
        return this._kernel;
    }
    get rendermime() {
        return this._rendermime;
    }
    /**
     * A signal emitted when state is restored to the widget manager.
     *
     * #### Notes
     * This indicates that previously-unavailable widget models might be available now.
     */
    get restored() {
        return this._restored;
    }
    /**
     * Whether the state has been restored yet or not.
     */
    get restoredStatus() {
        return this._restoredStatus;
    }
    register(data) {
        this._registry.set(data.name, data.version, data.exports);
    }
    /**
     * Get a model
     *
     * #### Notes
     * Unlike super.get_model(), this implementation always returns a promise and
     * never returns undefined. The promise will reject if the model is not found.
     */
    get_model(model_id) {
        const _super = Object.create(null, {
            get_model: { get: () => super.get_model }
        });
        return __awaiter(this, void 0, void 0, function* () {
            const modelPromise = _super.get_model.call(this, model_id);
            if (modelPromise === undefined) {
                throw new Error('widget model not found');
            }
            return modelPromise;
        });
    }
    /**
     * Register a widget model.
     */
    register_model(model_id, modelPromise) {
        super.register_model(model_id, modelPromise);
        // Update the synchronous model map
        modelPromise.then(model => {
            this._modelsSync.set(model_id, model);
            model.once('comm:close', () => {
                this._modelsSync.delete(model_id);
            });
        });
    }
    /**
     * Close all widgets and empty the widget state.
     * @return Promise that resolves when the widget state is cleared.
     */
    clear_state() {
        const _super = Object.create(null, {
            clear_state: { get: () => super.clear_state }
        });
        return __awaiter(this, void 0, void 0, function* () {
            yield _super.clear_state.call(this);
            this._modelsSync = new Map();
        });
    }
    /**
     * Synchronously get the state of the live widgets in the widget manager.
     *
     * This includes all of the live widget models, and follows the format given in
     * the @jupyter-widgets/schema package.
     *
     * @param options - The options for what state to return.
     * @returns Promise for a state dictionary
     */
    get_state_sync(options = {}) {
        const models = [];
        for (let model of this._modelsSync.values()) {
            if (model.comm_live) {
                models.push(model);
            }
        }
        return base_1.serialize_state(models, options);
    }
}
exports.WidgetManager = WidgetManager;
//# sourceMappingURL=widget-manager.js.map